#include "stdafx.h"

void clean_up() {
	// Shut down everything
	tunnel_shut_down();
	fire_shut_down();
	polygons_shut_down();
	plasma_shut_down();
	blobs_shut_down();
	shut_down_music();
	shut_down_console();
}

// Where do we respond to the music? (Measured in samples).
int synch_points[] = {0, 278000, 743424, 1113088, 1481728, 1848320, 2216960, 2587648, 2953216, 3321856, 3688448, 4055040, 4423680, 4795392, 5159936, 5528576, 5712896, 5897216, 6264832, 6632448, 7059456, 0};

int cur_timeline_pos = -1;
long sound_pos;

int get_fraction_through_section() {
	int divisor = synch_points[cur_timeline_pos+1]-synch_points[cur_timeline_pos];
	if (divisor) {
		return (0xFF*(sound_pos-synch_points[cur_timeline_pos]))/(divisor);
	} else {
		return 127;
	}
}

int main(int argc, _TCHAR* argv[]) {

	set_console_colour(15,4);
	printf("ASCII MADNESS II is loading...                                                  ");
	reset_console_colour();

	printf("Setting up console graphics system... ");
	if (!set_up_console()) {
		console_response(false);
		system("pause");
		clean_up();
		return 1;
	} else {
		console_response(true);
	}
	

	sprite title_screen = {0,};
	sprite end_credits = {0,};


	// INIT all the effects.
	if (!tunnel_init() ||
		!fire_init() ||
		!polygons_init() ||
		!plasma_init() ||
		!blobs_init() ||
		!load_sprite("data/title.spr",&title_screen) ||
		!load_sprite("data/end.spr",&end_credits))
	{
		console_warning();
		printf("Setting up effect failed.\n");
		system("pause");
		clean_up();
		return 1;
	}

	

	// Start up the music/FMOD/stuffs
	if (!init_music()) {
		console_warning();
		printf("Music failure - quitting!\n");
		system("pause");
		clean_up();
		return 1;
	}
	
	DWORD last_frame = GetTickCount();
	DWORD curr_frame = GetTickCount();
	DWORD last_fps = last_frame;

	int fps_count = 0;

	set_console_colour(15,2);
	printf("ASCII MADNESS II is ready to go...\t\t\t\t\t\tPress ALT+ENTER for full-screen mode, ESC to exit, SPACE to skip and F12 to\ttake a screenshot. Press SPACE now to start the demo...\t\t\t\t");
	reset_console_colour();

	DWORD start_time = GetTickCount();
	while ((!key_pressed[VK_SPACE]) && (GetTickCount()<start_time+7000)) {
		read_key_presses();
		Sleep(10);
	}

	for (int i=0; i<50; ++i) printf("\n");

	hide_cursor();
	FSOUND_PlaySound(0, background_music);

	while (!key_pressed[VK_ESCAPE] && cur_timeline_pos!=0xFF) {
	
		sound_pos = FSOUND_GetCurrentPosition(0);

		Sleep(1);

		// Handle time-difference stuff:
		curr_frame = GetTickCount();
		int timePassed = (int)(curr_frame-last_frame);
		++fps_count;

		// Do we need to draw our FPS counter?
		if (curr_frame-last_fps >= 1000) {
			char new_window_title[256];
			sprintf(new_window_title, "ASCII Madness II - %d FPS", fps_count);
			SetConsoleTitle(new_window_title);
			last_fps = curr_frame;
            fps_count = 0;
		}
		last_frame = curr_frame;		
		
		// Handle mouse/joystick events.
		read_key_presses();

		if (cur_timeline_pos!=-1) {

			// This is where we decide which effect to draw.
			if (cur_timeline_pos==0) {

				blit_sprite(title_screen, 0, 0);
				black_and_white_film(timePassed);
				disp_current_buffer();
			} else if (cur_timeline_pos==1) {
				int fade = get_fraction_through_section()*4;
				int e_fade = (256-get_fraction_through_section())*4;

				blit_sprite(title_screen, 0, 0);
				
				if (fade<256) {
					flip_buffers();
					blit_sprite(title_screen, 0, 0);
					black_and_white_film(timePassed);
					flip_buffers();
					disp_blended_buffers(0,fade);
				} else if (e_fade<256) {
					flip_buffers();
					polygons_draw_frame(timePassed);
					flip_buffers();
					disp_blended_buffers(1,e_fade);
				} else {
					disp_current_buffer();
				}
				 
			} else if (cur_timeline_pos==2||cur_timeline_pos==5) {
				polygons_draw_frame(timePassed);
				disp_current_buffer();
			} else if (cur_timeline_pos==3) {

				int s_fade = get_fraction_through_section()*6;

				if (s_fade<256) {
					flip_buffers();
					polygons_draw_frame(timePassed);
					flip_buffers();
					fire_draw_frame(timePassed);
					disp_blended_buffers(1,s_fade);
				} else {
					fire_draw_frame(timePassed);
					disp_current_buffer();
				}

			} else if (cur_timeline_pos==4) {
				polygons_draw_frame(timePassed);
				flip_buffers();
				fire_draw_frame(timePassed);
				flip_buffers();
				disp_blended_buffers(1,127);
				
			} else if (cur_timeline_pos==6) {

				int s_fade = get_fraction_through_section()*3;
				int e_fade = (256-get_fraction_through_section())*3;

				if (s_fade<256) {
					flip_buffers();
					polygons_draw_frame(timePassed);
					flip_buffers();
					tunnel_draw_frame(timePassed);
					disp_blended_buffers(2,s_fade);
				} else {
					tunnel_draw_frame(timePassed);
					disp_current_buffer();
				}

			} else if (cur_timeline_pos==7) {
				tunnel_draw_frame(timePassed);
				black_and_white_film(timePassed);
				disp_current_buffer();
			} else if (cur_timeline_pos==8) {
				plasma_draw_frame(timePassed);
				disp_current_buffer();
			} else if (cur_timeline_pos==9||cur_timeline_pos==10) {

				int already_drawn_tunnel = 1;

				clear_current_buffer();
				if (cur_timeline_pos==10) {
					tunnel_draw_frame(timePassed);
					already_drawn_tunnel = 0;
					invert(timePassed);
				}
				flip_buffers();
				int f = get_fraction_through_section();
				int pt_a = 0;
				int pt_b = 64;
				int pt_c = 128;
				int pt_d = 192;

				if (cur_timeline_pos==10) {
					f = 255-f;
					pt_a = 256;
					pt_b = 192;
					pt_c = 128;
					pt_d = 64;
				}

				clear_current_buffer();
				
				
				if (f>=pt_a) {
					polygons_draw_frame(timePassed);
					send_to_corner(0);
				}
				if (f>=pt_b) {
					fire_draw_frame(timePassed);
					send_to_corner(3);				
				}
				if (f>=pt_c) {
					tunnel_draw_frame(timePassed*already_drawn_tunnel);
					send_to_corner(1);
				}
				if (f>=pt_d) {
					plasma_draw_frame(timePassed);
					send_to_corner(2);
				}

				if (f<32 && cur_timeline_pos==9) {
					plasma_draw_frame(timePassed);
					disp_blended_buffers(3,f*8);
				} else {
					flip_buffers();
					disp_current_buffer();
				}
			} else if (cur_timeline_pos==11) {
				tunnel_draw_frame(timePassed);
				invert(timePassed);
				flip_buffers();
				tunnel_draw_frame(0);
				disp_blended_buffers(1,get_fraction_through_section()*3);
			} else if (cur_timeline_pos==12) {

				tunnel_draw_frame(timePassed);
				flip_buffers();

				clear_current_buffer();
				blobs_draw_frame(timePassed);
				
				disp_blended_buffers(1,get_fraction_through_section()*3);
			} else if (cur_timeline_pos==13) {

				clear_current_buffer();
				blobs_draw_frame(timePassed);
				
				flip_buffers();

				plasma_draw_frame(timePassed);

				disp_blended_buffers(1,get_fraction_through_section());
			} else if (cur_timeline_pos==14||cur_timeline_pos==15) {
				
				clear_current_buffer();
				blobs_draw_frame(timePassed);
				send_to_corner(0);
				send_to_corner(3);
				invert(timePassed);
				send_to_corner(1);
				send_to_corner(2);
				
				if (cur_timeline_pos==14) {
					plasma_draw_frame(timePassed);
					flip_buffers();
					disp_blended_buffers(2,get_fraction_through_section()*4);
				} else {
					fire_draw_frame(timePassed);
					disp_blended_buffers(0,get_fraction_through_section()*4);
				}
			} else if (cur_timeline_pos==16) {
				fire_draw_frame(timePassed);
				disp_current_buffer();
			} else if (cur_timeline_pos==17) {
				fire_draw_frame(timePassed/2);
				flip_buffers();
				fire_draw_frame(0);
				black_and_white_film(timePassed);
				disp_blended_buffers(1,get_fraction_through_section()*2);
			} else if (cur_timeline_pos==18) {
				fire_draw_frame(timePassed/2);
				black_and_white_film(timePassed/2);
				flip_buffers();
				polygons_draw_frame(timePassed/2);            
				disp_blended_buffers(1,get_fraction_through_section()*2);
			} else if (cur_timeline_pos==19) {
				polygons_draw_frame(timePassed/2);
				flip_buffers();
				plasma_draw_frame(timePassed);
				blit_sprite(end_credits,15,2);
				disp_blended_buffers(1,get_fraction_through_section()*4);
			} else {
				plasma_draw_frame(timePassed);
				blit_sprite(end_credits,15,2);
				disp_current_buffer();
			}
		
		}


		// Handle screen-shottery stuff.
		if (key_pressed[VK_F12]) {
			CreateDirectory("screenshots", NULL);
			while (key_pressed[VK_F12]) read_key_presses();
			struct tm *newtime;
			__time64_t long_time;
			_time64(&long_time);
			newtime = _localtime64(&long_time);
			char filename[256];
			sprintf(filename, "screenshots/%d-%d-%d-%d-%d-%d.rtf",
				newtime->tm_year+1900, newtime->tm_mon, newtime->tm_mday,
				newtime->tm_hour, newtime->tm_min, newtime->tm_sec);
			save_screenshot(filename);
		}

		// Skip!
		if (key_pressed[VK_SPACE]) {
			while (key_pressed[VK_SPACE]) read_key_presses();
			FSOUND_SetCurrentPosition(0,synch_points[cur_timeline_pos+1]);
		}

		// Advance timeline		
		if (sound_pos>=synch_points[cur_timeline_pos+1]) {
			++cur_timeline_pos;
			if (synch_points[cur_timeline_pos+1]==0) {
				FSOUND_StopSound(0);
				sound_pos = 0;
			}
		}
	}

	clean_up();
	return 0;
}



