#include "stdafx.h"

typedef struct point_2D {
	int x;
	int y;
	double z;
	double u;
	double v;
	double nx;
	double ny;
	double nz;
} point_2D;

typedef struct point_3D {
	double x;
	double y;
	double z;
	double u;
	double v;
	double nx;
	double ny;
	double nz;
} point_3D;

typedef struct mesh {
	int num_triangles;
	point_3D *triangles;
	sprite *texture;
} mesh;

sprite globe_texture;

mesh sphere;

double l_pos = 0;

double r_x = 0;
double r_y = 0;

double depth_buffer[80*50];

void free_mesh(mesh mesh_to_free) {
	if (mesh_to_free.triangles) delete mesh_to_free.triangles;
}

bool load_mesh(char *filename, mesh *mesh_to_load_into) {
	FILE *ms = fopen(filename, "r");
	if (!ms) {
		return false;
	}
	fscanf(ms, "[%d]", &mesh_to_load_into->num_triangles);
	mesh_to_load_into->triangles = new point_3D[mesh_to_load_into->num_triangles*3];

	for (int i=0; i<mesh_to_load_into->num_triangles*3; ++i) {
		float x, y, z, u, v, nx, ny, nz;
		fscanf(ms,"(%f,%f,%f){%f,%f}(%f,%f,%f)",
			&x,&z,&y,
			&u,&v,
			&nx,&nz,&ny
		);
		mesh_to_load_into->triangles[i].x = (double)x;
		mesh_to_load_into->triangles[i].y = (double)y;
		mesh_to_load_into->triangles[i].z = (double)z;
		
		mesh_to_load_into->triangles[i].u = (double)u;
		mesh_to_load_into->triangles[i].v = (double)v;
		
		mesh_to_load_into->triangles[i].nx = (double)nx;
		mesh_to_load_into->triangles[i].ny = (double)ny;
		mesh_to_load_into->triangles[i].nz = (double)nz;
	}

	fclose(ms);
	return true;
}

void draw_span(
	int x0, int x1,
	double u0, double v0,
	double u1, double v1,
	double z0, double z1,
	int b0, int b1,
	int y, sprite tex) {
	
	if (x1 < x0) {
		int s = x0; x0 = x1; x1 = s;
		double t = u0; u0 = u1; u1 = t;
		t = v0; v0 = v1; v1 = t;
		s = b1; b1 = b0; b0 = s;
	}
	if (x0!=x1) {
	
		double texScale = 1/((double)x1-x0);
		int startX = x0;

	
		if (x0<0) x0=0;
		if (x0>79) x0=79;
		if (x1<0) x1=0;
		if (x1>79) x1=79;

		double texProg = (x0-startX)*texScale;


		int startPlot = x0+y*80;
		for (int x=x0; x<=x1; ++x) {

			double z = z0+texProg*(z1-z0);
			if (z<depth_buffer[x+80*y]) {
				depth_buffer[x+80*y] = z;
				int u = (int)((u0+texProg*(u1-u0))*(double)(tex.w));
				int v = (int)((v0+texProg*(v1-v0))*(double)(tex.h));
				while (u<0) u+= tex.w;
				while (v<0) v+= tex.h;
				u%=tex.w;
				v%=tex.h;


				int b = (int)((double)b0+texProg*(double)(b1-b0));
				if (b<0) b = 0;
				if (b>0xFF) b = 0xFF;

				
				short pix = colour_map[tex.data[u+v*tex.w]*256+b];

				console_buffer[startPlot].Attributes = palette[pix].Attributes;
				console_buffer[startPlot].Char.AsciiChar = palette[pix].Char.AsciiChar;
			}
			++startPlot;
			texProg+=texScale;
		}
	}
}

void clear_z_buffer() {
	for (int i=0; i<80*50; ++i) {
		depth_buffer[i] = 1E10;
	}
}

void fill_triangle(
	int x0, int y0, double z0, double u0, double v0, int b0,
	int x1, int y1, double z1, double u1, double v1, int b1,
	int x2, int y2, double z2, double u2, double v2, int b2,
	sprite tex)
{

	// Sort points
	int s;
	double t;
	if (y0 > y1) {
		s = x1; x1 = x0; x0 = s;
		s = y1; y1 = y0; y0 = s;
		t = u1; u1 = u0; u0 = t;
		t = v1; v1 = v0; v0 = t;
		t = z1; z1 = z0; z0 = t;
		s = b1; b1 = b0; b0 = s;
	}
	if (y0 > y2) {
		s = x2; x2 = x0; x0 = s;
		s = y2; y2 = y0; y0 = s;
		t = u2; u2 = u0; u0 = t;
		t = v2; v2 = v0; v0 = t;
		t = z2; z2 = z0; z0 = t;
		s = b2; b2 = b0; b0 = s;
	}
	if (y1 > y2) {
		s = x2; x2 = x1; x1 = s;
		s = y2; y2 = y1; y1 = s;
		t = u2; u2 = u1; u1 = t;
		t = v2; v2 = v1; v1 = t;
		t = z2; z2 = z1; z1 = t;
		s = b2; b2 = b1; b1 = s;
	}

	if (y0!=y1 && y2!=y0) {
		double fracYAI = (double)1/(double)(y1-y0);
		double fracYBI = (double)1/(double)(y2-y0);
		double fracYA = 0;
		double fracYB = 0;
		for (int y = y0; y<y1; ++y) {
			if (y>=0 && y<50) {
				int   xA = (int)(x0+fracYA*(double)(x1-x0));
				double uA = u0+fracYA*(double)(u1-u0);
				double vA = v0+fracYA*(double)(v1-v0);
				double zA = z0+fracYA*(double)(z1-z0);
				int   bA = (int)(b0+fracYA*(double)(b1-b0));

				int   xB = (int)(x0+fracYB*(double)(x2-x0));
				double uB = u0+fracYB*(double)(u2-u0);			
				double vB = v0+fracYB*(double)(v2-v0);
				double zB = z0+fracYB*(double)(z2-z0);
				int   bB = (int)(b0+fracYB*(double)(b2-b0));			

				draw_span(xA,xB,uA,vA,uB,vB,zA,zB,bA,bB,y,tex);
			}
			fracYA+=fracYAI;
			fracYB+=fracYBI;
		}
	}

	if (y1!=y2 && y2!=y0) {
		double fracYAI = (double)1/(double)(y2-y1);
		double fracYBI = (double)1/(double)(y2-y0);
		double fracYA = 0;
		double fracYB = (double)(y1-y0)*fracYBI;
		for (int y = y1; y<=y2; ++y) {
			if (y>=0 && y<50) {
				int   xA = (int)(x1+fracYA*(double)(x2-x1));
				double uA = u1+fracYA*(double)(u2-u1);
				double vA = v1+fracYA*(double)(v2-v1);
				double zA = z1+fracYA*(double)(z2-z1);
				int   bA = (int)(b1+fracYA*(double)(b2-b1));

				int   xB = (int)(x0+fracYB*(double)(x2-x0));		
				double uB = u0+fracYB*(double)(u2-u0);			
				double vB = v0+fracYB*(double)(v2-v0);
				double zB = z0+fracYB*(double)(z2-z0);
				int   bB = (int)(b0+fracYB*(double)(b2-b0));

				draw_span(xA,xB,uA,vA,uB,vB,zA,zB,bA,bB,y,tex);
			}
			fracYA+=fracYAI;
			fracYB+=fracYBI;
		}
	}
}



void draw_mesh(mesh mesh_to_draw, double r_x, double r_y) {
	point_2D translated_points[3];
	int brightnesses[3];

	point_3D light;
	light.nx = (-sin(l_pos*2));
	light.ny = (-sin(l_pos/2));
	light.nz = (-cos(l_pos*2));

	double globe_dist = 64 + 64 * cos(l_pos/2);

	/*CONSOLE_FONT_INFO cur_font;
	GetCurrentConsoleFont(GetStdHandle(STD_OUTPUT_HANDLE),FALSE,&cur_font);*/

	/*double x_scaler = (double)cur_font.dwFontSize.X/(double)cur_font.dwFontSize.Y;
	double y_scaler = (double)cur_font.dwFontSize.Y/(double)cur_font.dwFontSize.X;*/


	for (int i=0; i<mesh_to_draw.num_triangles; ++i) {
		for (int p=0; p<3; ++p) {
			int t = i*3+p;
			double rotX1 = -mesh_to_draw.triangles[t].x * sin(r_x) + mesh_to_draw.triangles[t].y * cos(r_x);
			double rotY1 = -mesh_to_draw.triangles[t].x * cos(r_x) * sin(r_y) - mesh_to_draw.triangles[t].y * sin(r_x) * sin(r_y) - mesh_to_draw.triangles[t].z * cos(r_y);
			double rotZ1 = -mesh_to_draw.triangles[t].x * cos(r_x) * cos(r_y) - mesh_to_draw.triangles[t].y * sin(r_x) * cos(r_y) + mesh_to_draw.triangles[t].z * sin(r_y);

			translated_points[p].nx = -mesh_to_draw.triangles[t].nx * sin(r_x) + mesh_to_draw.triangles[t].ny * cos(r_x);
			translated_points[p].ny = -mesh_to_draw.triangles[t].nx * cos(r_x) * sin(r_y) - mesh_to_draw.triangles[t].ny * sin(r_x) * sin(r_y) - mesh_to_draw.triangles[t].nz * cos(r_y);
			translated_points[p].nz = -mesh_to_draw.triangles[t].nx * cos(r_x) * cos(r_y) - mesh_to_draw.triangles[t].ny * sin(r_x) * cos(r_y) + mesh_to_draw.triangles[t].nz * sin(r_y);

			translated_points[p].x = (int)(9 * 32 * (rotX1 / (rotZ1 + 128 + globe_dist)) + 40);
			translated_points[p].y = (int)(8 * 32 * (rotY1 / (rotZ1 + 128 + globe_dist)) + 25);
			translated_points[p].u = mesh_to_draw.triangles[t].u;
			translated_points[p].v = mesh_to_draw.triangles[t].v;
			translated_points[p].z = rotZ1;	

			point_3D norm_light;
			double scale_factor = pow(pow(light.nx,2)+pow(light.ny,2)+pow(light.nz,2),0.5);
			if (scale_factor!=0) {
				scale_factor = 1/scale_factor;
				norm_light.nx = light.nx * scale_factor;
				norm_light.ny = light.ny * scale_factor;
				norm_light.nz = light.nz * scale_factor;
			}

			double d_p = 
				norm_light.nx*translated_points[p].nx+
				norm_light.ny*translated_points[p].ny+
				norm_light.nz*translated_points[p].nz;

			brightnesses[p] = 255-(int)(255*d_p);

			if (brightnesses[p]<0) brightnesses[p] = 0;
			if (brightnesses[p]>230) brightnesses[p] = 230;

		}
		
		fill_triangle(
			translated_points[0].x,translated_points[0].y,translated_points[0].z,translated_points[0].u, translated_points[0].v,brightnesses[0],
			translated_points[1].x,translated_points[1].y,translated_points[1].z,translated_points[1].u, translated_points[1].v,brightnesses[1],
			translated_points[2].x,translated_points[2].y,translated_points[2].z,translated_points[2].u, translated_points[2].v,brightnesses[2],
			*mesh_to_draw.texture);

	}
}

bool polygons_init() {
	if (!load_sprite("data/earth.spr", &globe_texture)) {
		return false;
	}
	if (!load_mesh("data/sphere.msh", &sphere)) {
		return false;
	}
	sphere.texture = &globe_texture;

	return true;
}
bool polygons_shut_down() {
	free_sprite(globe_texture);
	free_mesh(sphere);
	return true;
}

void polygons_draw_frame(int time) {
	clear_current_buffer();
	clear_z_buffer();
	draw_mesh(sphere, r_x, r_y);
	r_x += ((double)time)/1800;
	r_y = -sin(r_x/10);
	l_pos+=((double)time)/800;
}